'-- VA18B.bas -------------------------------------------------- >>fst'14<< -
' - dll to access a multimeter VA18B (Mastech)
'  - can open/close serial port
'  - can get number of bytes waiting in serial buffer
'  - can get data from multimeter
' - compiled with FreeBasic 0.24 (parameter -dll)
'-- V 20140211 ------------------------------- http://www.FrankSteinberg.de -

Extern "Windows-MS"

Declare Function _Value(bRec() As UByte) As String
Declare Function _AcDc(bRec() As UByte) As String
Declare Function _Unit(bRec() As UByte) As String
Declare Function _Info(bRec() As UByte) As String
Declare Sub _FlushBuffer()

Dim Shared As String DMM_Value, DMM_Unit, DMM_AcDc, DMM_Info, sError
 
Function Comport_Auto() As UInteger Export
'Tries to open the highest available Comport. 
'Success: Returns comport number.
'Fail:    Returns 0

 Dim As String sPortNr
 Dim As UInteger iPortErr, iPortNr 
 Dim As UByte iTry

 For iTry = 1 To 20      'try to open a comport for max. 20 times 
	 For iPortNr = 256 To 1 Step -1                  'possible comports are Com1 to Com256
	  sPortNr="COM" +Trim(Str(iPortNr),Any Chr(32))  'build comport-string
	  'try to open comport:
	  iPortErr = Open COM (sPortNr + ":2400,N,8,1,BIN,Cs0,Ds0,Cd0,RS" For Binary As #1)
	  If iPortErr = 0 Then  'if available ...
	   _FlushBuffer         'remove old data from comport-buffer
	  	Return iPortNr       'return portnumber
	  	Exit Function        'exit function
	  EndIf
	 Next iPortNr           'try next portnumber
	 Sleep(100)             'if no port found, wait 0.1 Sec ...
 Next iTry               'and try again
  
 Return 0                'return 0 if all attempts failed
 	
End Function
 
Function Comport_Open(iPortNr As UInteger) As UInteger Export
'Tries to open given Comport. 
'Success: Returns comport number.
'Fail:    Returns 0

 Dim As String sPortNr
 Dim As UInteger iPortErr
 Dim As UByte iTry

 For iTry = 1 To 20      'try to open given comport for max. 20 times 
	  sPortNr="COM" +Trim(Str(iPortNr),Any Chr(32))  'build comport-string
	  'try to open comport:
	  iPortErr = Open COM (sPortNr + ":2400,N,8,1,BIN,Cs0,Ds0,Cd0,RS" For Binary As #1)
	  If iPortErr = 0 Then  'if available ...
	   _FlushBuffer         'remove old data from comport-buffer
	  	Return iPortNr       'return portnumber
	  	Exit Function        'exit function
	  EndIf
	 Sleep(100)             'if no port found, wait 0.1 Sec ...
 Next iTry               'and try again
  
 Return 0                'return 0 if all attempts failed
 	
End Function

Function Comport_Close() As UInteger Export
'Tries to close previous opened Comport. 
'Success: Returns 0
'Fail:    Returns non-zero
	
	Return Close(#1)
	
End Function

Function Comport_Buffer() As UInteger Export
'Returns the number of unread bytes in comport-buffer.	
	
	Return LOC(1)
	
End Function

Sub DMM_Data(DMM_Value As ZString Ptr, DMM_Unit As ZString Ptr, DMM_AcDc As ZString Ptr, DMM_Info As ZString Ptr) Export
'Receives 14 databytes from comport an build a value as string (allways six bytes long).	

 Dim bRec(1 TO 14) As UByte 
 Dim As UByte i, Test
 Dim As Double TStart
 'Dim As String sError

 sError = ""
 TStart = Timer                 'start timeout timer
  
 '*** Testing for startbyte (HIGHnibble = 1):
 Do
 	If LOC(1) Then                'if data in comport-buffer ...
   Get #1,,bRec(1)														'get one byte from comport
   Test = bRec(1) \ 16          'get HighNibble
 	Else
 		Sleep(50)                    'wait a little if no data from comport
 	EndIf
  If Timer-TStart > 4 Then      'timeout handling
   	sError = "* TimeOut! Unable to detect Byte1 "
   	GoTo Results
  EndIf
 Loop Until Test = 1            'repeat until find first databyte
 
 '*** Receive databyte 2-14: 
 For i = 2 TO 14                'get byte 2 - 14
 	While Loc(1) = 0              'loop until data in comport buffer
   Sleep(10)                    'wait a little if no data from comport
   If Timer-TStart > 5 Then     'timeout handling
   	 sError = "* TimeOut! Unable to detect Byte " + Str(i) + " "
   	 GoTo Results
   EndIf
 	Wend
  Get #1,,bRec(i)              'if data in comport-buffer ...
  Test = bRec(i) \ 16          'get HighNibble
  If Test <> i Then            'check if HighNibble fits to place in dataframe
    sError = "* Error in Byte" + Str(i) + " = " + Hex(bRec(i)) + " "
    GoTo Results
  EndIf
 Next i
 
 Results:
  If Len(sError) Then 
   *DMM_Value = ""
   *DMM_Unit  = ""
   *DMM_AcDc  = ""
   *DMM_Info  = sError
   Exit Sub
  End If
  
  *DMM_Value = _Value(bRec())
  If Len(sError) Then 
  	*DMM_Value = ""
  EndIf
  *DMM_Unit  = _Unit(bRec())
  *DMM_AcDc  = _AcDc(bRec())
  *DMM_Info  = sError + _Info(bRec())
 
End Sub

Function _Value(bRec() As UByte) As String  'Make DMM Value from Byte 2 - 9 (string, 6 chars)
 
 Dim As UByte i, bPCount
 Dim bTwoNibbles As UInteger
 Dim Digit As String
 Dim sValue As String*6
 
 '*** Build a four digit value (as string):                            
 For i = 2 To 9 Step 2
  '** Create a 'new' Byte from two LowNibbles:
  bTwoNibbles = (bRec(i) Mod 16) + ((bRec(i+1) Mod 16) * 16)
	
	'       Seven Segment Display 
	'
	'             Bit0(1)
 '           XXXX 1 XXXX
	'          X           X
 '          X           X
 ' Bit1(2)  6           2  Bit4(16)
 '          X           X
 '          X  Bit5(32) X
 '           XXXX 7 XXXX
 '          X           X
 '          X           X
 ' Bit2(4)  5           3  Bit6(64)
 '          X           X
 '          X  Bit7(128)X
 '           XXXX 4 XXXX
 '       X
 '         Decimalpoint Bit3(8)  

  '** Make a digit from 'new' byte:	
		Select Case bTwoNibbles
			Case 215: Digit =  "0" 
			Case 223: Digit = ".0" : If i > 2 Then bPCount += 1    'count number of decimal points
			Case  80: Digit =  "1"
			Case  88: Digit = ".1" : If i > 2 Then bPCount += 1    '           -*-
			Case 181: Digit =  "2"
			Case 189: Digit = ".2" : If i > 2 Then bPCount += 1    '           -*-
			Case 241: Digit =  "3"
			Case 249: Digit = ".3" : If i > 2 Then bPCount += 1    '           -*-
			Case 114: Digit =  "4"
			Case 122: Digit = ".4" : If i > 2 Then bPCount += 1    '           -*-
			Case 227: Digit =  "5"
			Case 235: Digit = ".5" : If i > 2 Then bPCount += 1    '           -*-
			Case 231: Digit =  "6"
			Case 239: Digit = ".6" : If i > 2 Then bPCount += 1    '           -*-
			Case  81: Digit =  "7"
			Case  89: Digit = ".7" : If i > 2 Then bPCount += 1    '           -*-
			Case 247: Digit =  "8"
			Case 255: Digit = ".8" : If i > 2 Then bPCount += 1    '           -*-
			Case 243: Digit =  "9"
			Case 251: Digit = ".9" : If i > 2 Then bPCount += 1    '           -*-
			Case 134: Digit =  "L" : sError = "OL!  "              'treat OL display like an error
			Case 142: Digit = ".L" : sError = "OL!  "              '           -*-
			Case   0: Digit =  " "
			Case   8: Digit = ". " : If i > 2 Then bPCount += 1
			Case Else: Digit = "?" : sError += "* Unknown sign! "  'unknown byte is an error too
		End Select
		'Error, if more than one decimal point in dataframe:
		If bPCount > 1 Then sError += "* Illegal decimal point! "
  sValue = sValue + Digit        'build four digits
 Next

 If Left(sValue, 1) = "." Then   'mostleft digit has a '-' instead a '.'
 	 Mid(sValue, 1, 1) = "-"
 Else 
 	 sValue = " " + sValue         'add left space if no '-'
 EndIf
 If Right(sValue, 1) = Chr(0) Then sValue = " " + sValue  'add right space if no '.' in value 

 Return sValue 
                                                
End Function

Function _AcDc(bRec() As UByte) As String   'Make Ac- or DC string or two spaces (2 chars)

 Dim sValue As String * 2

 sValue = "  "
 If bRec(1) And 4 Then sValue = "DC"   'Byte1, Bit2
 If bRec(1) And 8 Then sValue = "Ac"   'Byte1, Bit4
 
 Return sValue
 
End Function

Function _Unit(bRec() As UByte) As String   'Make measure unit string (4 chars)

 Dim sU1 As String * 1
 Dim sU2 As String * 3
  
                        sU1 = " "    'space, if no sign
 If bRec(10) And 2 Then sU1 = "k"    'Byte10, Bit1
 If bRec(10) And 4 Then sU1 = "n"    'Byte10, Bit2
 If bRec(10) And 8 Then sU1 = ""    'Byte10, Bit3
 If bRec(11) And 2 Then sU1 = "M"    'Byte11, Bit2
 If bRec(11) And 8 Then sU1 = "m"    'Byte11, Bit4
 
                        sU2 = "???"  '... if unknown unit
 If bRec(11) And 4 Then sU2 = "%  "  'Byte11, Bit3
 If bRec(12) And 4 Then sU2 = "Ohm"  'Byte12, Bit2
 If bRec(12) And 8 Then sU2 = "F  "  'Byte12, Bit3
 If bRec(13) And 2 Then sU2 = "Hz "  'Byte13, Bit1
 If bRec(13) And 4 Then sU2 = "V  "  'Byte13, Bit2
 If bRec(13) And 8 Then sU2 = "A  "  'Byte13, Bit3
 If bRec(14) And 2 Then sU2 = "C "  'Byte14, Bit1
 'If bRec(14) And 4 Then sU2 = "*E2"  'Byte14, Bit2

 Return sU1 + sU2                    'build and return unit-string

End Function

Function _Info (bRec() As UByte) As String  'Make string from the other display-info or errormessage (variable number of chars)  

 Dim sValue As String
 
 If bRec(1)  And 2 Then 
 	 sValue = "Auto"                                    'Byte1,  Bit1
 Else
 	 'sValue = sValue + " Rang"                         'Byte1,  Bit1
 EndIf
 If bRec(10) And 1 Then sValue = sValue + " Dio"      'Byte10, Bit0
 If bRec(11) And 1 Then sValue = sValue + " Beep"     'Byte11, Bit0
 If bRec(12) And 1 Then sValue = sValue + " Hold"     'Byte12, Bit0
 If bRec(12) And 2 Then sValue = sValue + " Rel"      'Byte12, Bit1
 If bRec(13) And 1 Then sValue = sValue + " Batt"     'Byte13, Bit0
 If bRec(14) And 1 Then sValue = sValue + " MAX"      'Byte14, Bit0
 If bRec(14) And 8 Then sValue = sValue + " MIN"      'Byte14, Bit3
 'If bRec(1) And 1 Then sValue = sValue + " Pc-Link"  'Byte1,  Bit0

 sValue = LTrim(sValue)
 Return sValue

End Function

Sub _FlushBuffer()           'Remove all data from comport-buffer.
	
	Dim sBuf As String
	
	If LOC(1) Then	sBuf = Input(LOC(1), 1)

End Sub

End Extern